/*
 *  QHTernarySolve.h
 *  gibbs
 *
 *  Created by Thomas Cool on 11/4/09.
 *  Copyright 2009 Thomas Cool. All rights reserved.
 *
 */
typedef struct {
    double C2;     /*+ Ternary concentration variable
                    +latex+$C_2$ ($0\rightarrow1$)
                    -latex-2 (0 -> 1)
                    +*/
    double C3;     /*+ Ternary concentration variable
                    +latex+$C_3$ ($0\rightarrow1-C_2$)
                    -latex-3 (0 -> 1-C2)
                    +*/
    int efunc;     /*+ Index indicating which energy surface (phase) this point
                    is on; a negative value indicates that energy derivatives
                    are not yet calculated. +*/
    double T;      /*+ Temperature at which the free energies and derivatives
                    are valid +*/
    double P;      /*+ Pressure at which the free energies and derivatives are
                    valid, -1 if not valid +*/
    double G;      /*+ Free energy +*/
    double G2;     /*+ Free energy derivative
                    +latex+$\partial G/\partial C_2$
                    -latex-dG/dC2
                    +*/
    double G3;     /*+ Free energy derivative
                    +latex+$\partial G/\partial C_3$
                    -latex-dG/dC3
                    +*/
    double G22;    /*+ Free energy second derivative
                    +latex+$\partial^2 G/\partial C_2^2$
                    +html+ d<sup>2</sup>G/dC2<sup>2</sup>
                    +*/
    double G33;    /*+ Free energy second derivative
                    +latex+$\partial^2 G/\partial C_3^2$
                    +html+ d<sup>2</sup>G/dC3<sup>2</sup>
                    +*/
    double G23;    /*+ Free energy mixed partial second derivative
                    +latex+$\partial^2 G/\partial C_2\partial C_3$
                    +html+ d<sup>2</sup>G/dC2dC3
                    +*/
} energy_point;

typedef struct {
    char *name;
    double C2; /*+ Composition at center of this Gaussian term +*/
    double C3; /*+ Composition at center of this Gaussian term +*/
    double G;  /*+ Free energy addition at the Gaussian center +*/
    double w;  /*+ Width of the Gaussian distribution +*/
    double n;  /*+ Exponent of the distribution, currently fixed at 2 +*/
} energy_gaussian;

typedef struct {
    char *name;      /*+ Name of this phase +*/
    double R;        /*+ Ideal gas law constant +*/
    double T0;       /*+ Reference temperature +*/
    double G1_T0;    /*+ Free energy of species 1 at reference temperature +*/
    double G2_T0;    /*+ Free energy of species 2 at reference temperature +*/
    double G3_T0;    /*+ Free energy of species 3 at reference temperature +*/
    double G1_C;     /*+ Species 1 heat capacity +*/
    double G2_C;     /*+ Species 2 heat capacity +*/
    double G3_C;     /*+ Species 3 heat capacity +*/
    double S1;       /*+ Species 1 entropy coefficient (inverse Flory-Huggins relative molar volume) +*/
    double S2;       /*+ Species 2 entropy coefficient +*/
    double S3;       /*+ Species 3 entropy coefficient +*/
    double S4;       /*+ 1-C2 entropy coefficient for square system +*/
    double S5;       /*+ 1-C3 entropy coefficient for square system +*/
    double Omega12;  /*+ Species 1-2 regular solution interaction parameter +*/
    double Omega13;  /*+ Species 1-3 regular solution interaction parameter +*/
    double Omega23;  /*+ Species 2-3 regular solution interaction parameter +*/
    double Omega123; /*+ Species 1-2-3 regular solution interaction parameter +*/
    double Omega234; /*+ Species 2-3-4 regular solution interaction parameter +*/
    double Omega235; /*+ Species 2-3-5 regular solution interaction parameter +*/
    double Omega245; /*+ Species 2-4-5 regular solution interaction parameter +*/
    double Omega345; /*+ Species 3-4-5 regular solution interaction parameter +*/
    double Omega2345;/*+ Species 2-3-4-5 regular solution interaction parameter+*/
    energy_gaussian *gauss; /*+ Array of Gaussian energy parameter structs +*/
    int n_gauss;     /*+ Number of Gaussian energy terms +*/
} energy_params;   /*+ Free energy parameters structure +*/

typedef enum {
    STANDARD=0,
    WITH_DERIVATIVES=1,
    NO_INFINITY=2,
    WITH_DERIVATIVES_NO_INFINITY=3
} free_energy_flags;

#define TERNARY_INFINITY HUGE_VALF
#define TERNARY_NEGATIVE_INFINITY (-HUGE_VALF)




typedef struct {
    double x;      /*+ Visualization
                    +latex+$x$-coordinate ($0\rightarrow1$)
                    -latex-x-coordinate (0 -> 1)
                    +*/
    double y;      /*+ Visualization
                    +latex+$y$-coordinate
                    -latex-y-coordinate
                    (scaled free energy) +*/
    double z;      /*+ Visualization
                    +latex+$z$-coordinate ($0\rightarrow\sqrt{3}/2$)
                    -latex-z-coordinate (0 -> sqrt(3)/2) +*/
    double red;    /*+ Visualization color red value
                    +latex+($0\rightarrow1$)
                    -latex-(0 -> 1)
                    +*/
    double green;  /*+ Visualization color green value
                    +latex+($0\rightarrow1$)
                    -latex-(0 -> 1)
                    +*/
    double blue;   /*+ Visualization color blue value
                    +latex+($0\rightarrow1$)
                    -latex-(0 -> 1)
                    +*/
    double alpha;  /*+ Visualization color alpha value
                    +latex+($0\rightarrow1$)
                    -latex-(0 -> 1)
                    +*/
} visual_point; /*+ Visualization "point" structure. +*/

typedef enum {
    ONE_PHASE_EDGE=0, /*+ One phase's boundary edges +*/
    TIE_SIMPLICES,    /*+ Tie lines between two phases, triangles between three,
                       etc. +*/
    SPINODAL          /*+ Spinodal boundary +*/
} phase_boundary_type;

typedef struct {
    int compos;   /*+ Number of points in a boundary edge +*/
    int *edges;   /*+ Edges with indices of energy_points comprising a phase
                   boundary; each edge has compos points +*/
    int n_edges;  /*+ Number of edges comprising this boundary +*/
    int *phases;  /*+ Phases involved (energy_params indices) starting with the
                   "current" phase +*/
    int n_phases; /*+ Number of phases involved +*/
    phase_boundary_type type; /*+ See phase_boundary_flags typedef enum +*/
} phase_boundary; /*+ Phase boundary structure which can store either the
                   boundary between a single phase and a multi-phase region,
                   or the tie lines/simplices across a multi-phase region +*/

typedef enum {
    ONE_PHASE=0,
    CRITICAL_POINT,
    TWO_PHASE,
    THREE_PHASE,
    ALL_TYPES=127
} facet_type;

typedef struct {
    int vertex [3];
    facet_type type;
} hull_facet;
int hullCalculate
(int dim, energy_point *points, int numpoints, energy_params *eparams,
 int nparams, double T, double P, hull_facet **facets, int *numfacets);
int hullReturnPhaseBoundaries
(energy_point *points, int n_points, hull_facet *facets, int n_facets,
 energy_params *eparams, double T, double P,
 phase_boundary **boundaries, int *n_bounds);
